@@ -18,7 +18,7 @@ from __future__ import print_function
 import logging
 import traceback
 
-from ._logging import setup, setup_http_handler
+from ._logging import _setup_comet_http_handler, _setup_comet_logging
 from ._reporting import (
     EXPERIMENT_CREATED,
     EXPERIMENT_CREATION_DURATION,
@@ -26,6 +26,13 @@ from ._reporting import (
 )
 from ._typing import Any, Dict, Optional
 from .api import API, APIExperiment
+from .artifacts import (
+    Artifact,
+    LoggedArtifact,
+    _get_artifact,
+    _log_artifact,
+    _parse_artifact_name,
+)
 from .comet import Streamer, format_url, generate_guid, is_valid_experiment_key
 from .config import (
     get_api_key,
@@ -33,10 +40,12 @@ from .config import (
     get_global_experiment,
     get_previous_experiment,
     get_ws_url,
+    init,
 )
 from .confusion_matrix import ConfusionMatrix
 from .connection import (
     INITIAL_BEAT_DURATION,
+    RestApiClient,
     RestServerConnection,
     WebSocketConnection,
     get_backend_address,
@@ -44,7 +53,13 @@ from .connection import (
     get_rest_api_client,
     log_url,
 )
-from .exceptions import BadCallbackArguments, InvalidAPIKey
+from .exceptions import (
+    BadCallbackArguments,
+    ExperimentCleaningException,
+    ExperimentDisabledException,
+    ExperimentNotAlive,
+    InvalidAPIKey,
+)
 from .experiment import BaseExperiment
 from .feature_toggles import HTTP_LOGGING, FeatureToggles
 from .json_encoder import NestedEncoder
@@ -63,7 +78,10 @@ from .loggers.xgboost_logger import patch as xg_patch
 from .logging_messages import (
     ADD_SYMLINK_ERROR,
     ADD_TAGS_ERROR,
+    ARTIFACT_PREVIEW_FEATURE_FLAG,
     EXPERIMENT_LIVE,
+    GET_ARTIFACT_VERSION_OR_ALIAS_GIVEN_TWICE,
+    GET_ARTIFACT_WORKSPACE_GIVEN_TWICE,
     INTERNET_CONNECTION_ERROR,
     INVALID_API_KEY,
     ONLINE_EXPERIMENT_THROTTLED,
@@ -79,6 +97,7 @@ from .utils import (
     Histogram,
     get_comet_version,
     get_time_monotonic,
+    make_template_filename,
     merge_url,
     valid_ui_tabs,
 )
@@ -97,6 +116,7 @@ __all__ = [
     "Histogram",
     "OfflineExperiment",
     "Optimizer",
+    "init",
 ]
 __version__ = get_comet_version()
 
@@ -120,7 +140,7 @@ if not get_config("comet.disable_auto_logging"):
     MODULE_FINDER.start()
 
 # Configure the logging
-setup(get_config())
+_setup_comet_logging(get_config())
 
 
 def start():
@@ -267,7 +287,7 @@ class Experiment(BaseExperiment):
 
         self.ws_connection = None  # type: Optional[WebSocketConnection]
         self.connection = None  # type: Optional[RestServerConnection]
-        self.rest_api_client = None
+        self.rest_api_client = None  # type: Optional[RestApiClient]
 
         if self.disabled is not True:
             self._start()
@@ -288,7 +308,7 @@ class Experiment(BaseExperiment):
             LOGGER.debug("Do not setup http logger, disabled by feature toggle")
             return
 
-        self.http_handler = setup_http_handler(
+        self.http_handler = _setup_comet_http_handler(
             log_url(get_backend_address()), self.api_key, self.id
         )
 
@@ -386,6 +406,7 @@ class Experiment(BaseExperiment):
             self.run_id,
             self.project_id,
             self.rest_api_client,
+            self.config.get_int(None, "comet.internal.file_upload_worker_ratio"),
             self._on_pending_rpcs_callback,
             self.config["comet.timeout.cleaning"],
             self.config["comet.timeout.upload"],
@@ -450,6 +471,11 @@ class Experiment(BaseExperiment):
         except Exception:
             LOGGER.debug("Failed to check experiment metadata", exc_info=True)
 
+        if self.streamer is not None and self.streamer.has_failed():
+            raise ExperimentCleaningException(
+                "Failed to successfully send all of the artifact's assets"
+            )
+
         return successful_clean
 
     def _check_experiment_throttled(self):
@@ -627,7 +653,7 @@ class Experiment(BaseExperiment):
         image_transparent_color=None,
         image_background_color_function=None,
         title="Comet Embedding",
-        template_filename="template_projector_config.json",
+        template_filename=None,
         group=None,
     ):
         """
@@ -646,6 +672,7 @@ class Experiment(BaseExperiment):
                 index, and returns a (red, green, blue) color tuple
             title: (optional) name of tensor
             template_filename: (optional) name of template JSON file
+            group: (optional) name of group of embeddings
 
         See also: `Experiment._log_embedding_list()` and `comet_ml.Embedding`
 
@@ -719,10 +746,136 @@ class Experiment(BaseExperiment):
         else:
             # Log the template:
             template = {"embeddings": [embedding.to_json()]}
+            if template_filename is None:
+                template_filename = make_template_filename()
+
             return self._log_asset_data(
                 template, template_filename, asset_type="embeddings"
             )
 
+    def log_artifact(self, artifact):
+        # type: (Artifact) -> LoggedArtifact
+        """
+        Log an Artifact object, synchronously create a new Artifact Version and upload
+        asynchronously all local and remote assets attached to the Artifact object.
+
+        Args:
+            artifact: an Artifact object
+
+        Returns: a [LoggedArtifact](/docs/python-sdk/LoggedArtifact/)
+        """
+
+        if self.disabled:
+            raise ExperimentDisabledException(
+                "Experiment %r is disabled, cannot log artifact" % self
+            )
+        elif not self.alive:
+            raise ExperimentNotAlive(
+                "Experiment %r is not alive, cannot log artifact" % self
+            )
+
+        if not isinstance(artifact, Artifact):
+            raise ValueError("%r is not an Artifact and cannot be logged" % artifact)
+
+        if not self.config["comet.override_feature.artifacts"]:
+            LOGGER.warning(ARTIFACT_PREVIEW_FEATURE_FLAG)
+
+        return _log_artifact(artifact, self)
+
+    def get_artifact(
+        self, artifact_name, workspace=None, version_or_alias=None,
+    ):
+        # type: (str, Optional[str], Optional[str]) -> LoggedArtifact
+        """ Returns a logged artifact object that can be used to access the artifact version assets and
+        download them locally.
+
+        If no version or alias is provided, the latest version for that artifact is returned.
+
+        Args:
+            artifact_name: Retrieve an artifact with that name. This could either be a fully
+                qualified artifact name like `workspace/artifact-name:versionOrAlias` or just the name
+                of the artifact like `artifact-name`.
+            workspace: Retrieve an artifact belonging to that workspace
+            version_or_alias: Optional. Retrieve the artifact by the given alias or version.
+
+        Returns: the LoggedArtifact
+        For example:
+
+        ```python
+        logged_artifact = experiment.get_artifact("workspace/artifact-name:version_or_alias")
+        ```
+
+        Which is equivalent to:
+
+        ```python
+        logged_artifact = experiment.get_artifact(
+            artifact_name="artifact-name",
+            workspace="workspace",
+            version_or_alias="version_or_alias")
+        ```
+        """
+        if self.disabled:
+            raise ExperimentDisabledException(
+                "Experiment %r is disabled, cannot get artifact" % self
+            )
+        elif not self.alive:
+            raise ExperimentNotAlive(
+                "Experiment %r is not alive, cannot get artifact" % self
+            )
+
+        if not self.config["comet.override_feature.artifacts"]:
+            LOGGER.warning(ARTIFACT_PREVIEW_FEATURE_FLAG)
+
+        # Parse the artifact_name
+        parsed_workspace, artifact_name, parsed_version_or_alias = _parse_artifact_name(
+            artifact_name
+        )
+
+        params = {}
+
+        if parsed_workspace is None and workspace is None:
+            # In that case, the backend will use the experiment id to get the workspace
+            param_workspace = None
+        elif parsed_workspace is not None and workspace is not None:
+            if parsed_workspace != workspace:
+                LOGGER.warning(
+                    GET_ARTIFACT_WORKSPACE_GIVEN_TWICE
+                    % (parsed_workspace, artifact_name)
+                )
+            param_workspace = workspace
+        elif workspace is None:
+            param_workspace = parsed_workspace
+        else:
+            param_workspace = workspace
+
+        if parsed_version_or_alias is not None and version_or_alias is not None:
+            if parsed_version_or_alias != version_or_alias:
+                LOGGER.warning(
+                    GET_ARTIFACT_VERSION_OR_ALIAS_GIVEN_TWICE
+                    % (parsed_version_or_alias, artifact_name)
+                )
+            param_version_or_alias = version_or_alias
+        elif parsed_version_or_alias is not None:
+            param_version_or_alias = parsed_version_or_alias
+        else:
+            param_version_or_alias = version_or_alias
+
+        params = {
+            "consumer_experiment_key": self.id,
+            "experiment_key": self.id,
+            "name": artifact_name,
+            "version_or_alias": param_version_or_alias,
+            "workspace": param_workspace,
+        }
+
+        logged_artifact = _get_artifact(
+            self.rest_api_client, params, self.id, self._summary, self.config
+        )
+
+        self._summary.increment_section("downloads", "artifacts")
+
+        return logged_artifact
+
 
 class ExistingExperiment(Experiment):
     """Existing Experiment allows you to report information to an
