@@ -17,8 +17,11 @@
 This module contains useful types and mirror the typing module
 
 """
+
+# isort: off
+
 from typing import *  # noqa
-from typing import IO, Union
+from typing import IO, BinaryIO, Union  # noqa
 
 
 class ValidFilePath(str):
