@@ -40,14 +40,9 @@ from .config import (
     DEFAULT_FILE_UPLOAD_READ_TIMEOUT,
     DEFAULT_STREAMER_MSG_TIMEOUT,
 )
-from .connection import format_messages_for_ws
+from .connection import FileUploadManager, format_messages_for_ws
 from .exceptions import CometRestApiException
-from .file_uploader import (
-    is_user_text,
-    upload_file_like_thread,
-    upload_file_thread,
-    upload_remote_asset_thread,
-)
+from .file_uploader import is_user_text
 from .json_encoder import NestedEncoder
 from .logging_messages import (
     CLOUD_DETAILS_MSG_SENDING_ERROR,
@@ -181,6 +176,7 @@ class Streamer(BaseStreamer):
         run_id,
         project_id,
         rest_api_client,
+        worker_cpu_ratio,
         pending_rpcs_callback=None,
         msg_waiting_timeout=DEFAULT_STREAMER_MSG_TIMEOUT,
         file_upload_waiting_timeout=ADDITIONAL_STREAMER_UPLOAD_TIMEOUT,
@@ -205,7 +201,7 @@ class Streamer(BaseStreamer):
         self.file_upload_waiting_timeout = file_upload_waiting_timeout
         self.file_upload_read_timeout = file_upload_read_timeout
 
-        self.upload_threads = []  # type: List[threading.Thread]
+        self.file_upload_manager = FileUploadManager(worker_cpu_ratio)
 
         self.experiment_key = experiment_key
         self.api_key = api_key
@@ -287,58 +283,61 @@ class Streamer(BaseStreamer):
         # Compute the url from the upload type
         url = self.connection.get_upload_url(message.upload_type)
 
-        upload_thread = upload_file_thread(
-            project_id=self.project_id,
+        self.file_upload_manager.upload_file_thread(
+            additional_params=message.additional_params,
+            api_key=self.api_key,
+            clean=message.clean,
+            critical=message._critical,
             experiment_id=self.experiment_key,
             file_path=message.file_path,
-            upload_endpoint=url,
-            api_key=self.api_key,
-            additional_params=message.additional_params,
             metadata=message.metadata,
-            clean=message.clean,
+            project_id=self.project_id,
             timeout=self.file_upload_read_timeout,
+            upload_endpoint=url,
+            on_asset_upload=message._on_asset_upload,
+            on_failed_asset_upload=message._on_failed_asset_upload,
         )
-        self.upload_threads.append(upload_thread)
         LOGGER.debug("Processing uploading message done")
-        LOGGER.debug("Upload threads %s", self.upload_threads)
 
     def _process_upload_in_memory_message(self, message):
         # type: (UploadInMemoryMessage) -> None
         # Compute the url from the upload type
         url = self.connection.get_upload_url(message.upload_type)
 
-        upload_thread = upload_file_like_thread(
-            project_id=self.project_id,
+        self.file_upload_manager.upload_file_like_thread(
+            additional_params=message.additional_params,
+            api_key=self.api_key,
+            critical=message._critical,
             experiment_id=self.experiment_key,
             file_like=message.file_like,
-            upload_endpoint=url,
-            api_key=self.api_key,
-            additional_params=message.additional_params,
             metadata=message.metadata,
+            project_id=self.project_id,
             timeout=self.file_upload_read_timeout,
+            upload_endpoint=url,
+            on_asset_upload=message._on_asset_upload,
+            on_failed_asset_upload=message._on_failed_asset_upload,
         )
-        self.upload_threads.append(upload_thread)
-        LOGGER.debug("Processing uploading message done")
-        LOGGER.debug("Upload threads %s", self.upload_threads)
+        LOGGER.debug("Processing in-memory uploading message done")
 
     def _process_upload_remote_asset_message(self, message):
         # type: (RemoteAssetMessage) -> None
         # Compute the url from the upload type
         url = self.connection.get_upload_url(message.upload_type)
 
-        upload_thread = upload_remote_asset_thread(
-            project_id=self.project_id,
-            experiment_id=self.experiment_key,
-            remote_uri=message.remote_uri,
-            upload_endpoint=url,
-            api_key=self.api_key,
+        self.file_upload_manager.upload_remote_asset_thread(
             additional_params=message.additional_params,
+            api_key=self.api_key,
+            critical=message._critical,
+            experiment_id=self.experiment_key,
             metadata=message.metadata,
+            project_id=self.project_id,
+            remote_uri=message.remote_uri,
             timeout=self.file_upload_read_timeout,
+            upload_endpoint=url,
+            on_asset_upload=message._on_asset_upload,
+            on_failed_asset_upload=message._on_failed_asset_upload,
         )
-        self.upload_threads.append(upload_thread)
-        LOGGER.debug("Processing uploading message done")
-        LOGGER.debug("Upload threads %s", self.upload_threads)
+        LOGGER.debug("Processing remote uploading message done")
 
     def _process_ws_message(self, message, offset):
         # type: (Message, int) -> None
@@ -515,8 +514,9 @@ class Streamer(BaseStreamer):
         # From now on, stop processing the message queue as it might contains file upload messages
         # TODO: Find a correct way of testing it
         self.stop_processing = True
+        self.file_upload_manager.close()
 
-        if not self._is_file_upload_done():
+        if not self.file_upload_manager.all_done():
             msg = (
                 "Waiting for completion of the file uploads (may take several seconds)"
             )
@@ -526,17 +526,15 @@ class Streamer(BaseStreamer):
                 self.file_upload_waiting_timeout,
             )
             wait_for_empty(
-                self._is_file_upload_done,
+                self.file_upload_manager.all_done,
                 self.file_upload_waiting_timeout,
-                progress_callback=self._show_remaining_file_uploads,
+                progress_callback=self.file_upload_manager.log_remaining_uploads,
                 sleep_time=5,
             )
 
-        if not self._is_msg_queue_empty() or not self._is_file_upload_done():
+        if not self._is_msg_queue_empty() or not self.file_upload_manager.all_done():
             remaining = self.messages.qsize()
-            remaining_upload = len(
-                [thread for thread in self.upload_threads if thread.is_alive() is True]
-            )
+            remaining_upload = self.file_upload_manager.remaining_uploads()
             LOGGER.error(STREAMER_WAIT_FOR_FINISH_FAILED, remaining, remaining_upload)
 
             self.connection.report(
@@ -548,6 +546,8 @@ class Streamer(BaseStreamer):
 
             return False
 
+        self.file_upload_manager.join()
+
         return True
 
     def _is_msg_queue_empty(self):
@@ -567,24 +567,9 @@ class Streamer(BaseStreamer):
         remaining = self.messages.qsize()
         LOGGER.info("Uploading %d metrics, params and output messages", remaining)
 
-    def _is_file_upload_done(self):
-        finished = True
-
-        for thread in self.upload_threads:
-            thread_finished = thread.is_alive() is False
-            finished = finished and thread_finished
-
-        return finished
-
-    def _show_remaining_file_uploads(self):
-        remaining_upload_number = 0
-        for thread in self.upload_threads:
-            if thread.is_alive() is True:
-                remaining_upload_number += 1
-
-        LOGGER.info(
-            "Still uploading %d file(s)", remaining_upload_number,
-        )
+    def has_failed(self):
+        # type: (...) -> bool
+        return self.file_upload_manager.has_failed()
 
 
 def compact_json_dump(data, fp):
@@ -788,7 +773,7 @@ class OfflineStreamer(BaseStreamer):
         if not self.messages.empty():
             remaining = self.messages.qsize()
             LOGGER.info(
-                "Comet failed to send all the data back (%s messages)" % remaining
+                "Comet failed to send all the data back (%s messages)", remaining
             )
 
         # Also wait for the thread to finish to be sure that all messages are
