@@ -18,6 +18,7 @@ import io
 import logging
 import os
 import os.path
+import shutil
 
 import six
 from everett.manager import (
@@ -32,8 +33,15 @@ from everett.manager import (
 )
 
 from ._logging import _get_comet_logging_config
-from ._typing import Any, Dict, List, Optional
-from .utils import log_once_at_level
+from ._typing import Any, Dict, List, Optional, Tuple, Union
+from .exceptions import InvalidRestAPIKey
+from .logging_messages import (
+    API_KEY_CHECK_FAILED,
+    API_KEY_IS_INVALID,
+    API_KEY_IS_NOT_SET,
+    API_KEY_IS_VALID,
+)
+from .utils import get_api_key_from_user, is_interactive, log_once_at_level
 
 try:  # everett version 1.0.1 or greater
     from everett.ext.inifile import ConfigIniEnv
@@ -58,9 +66,9 @@ DEBUG = False
 # Global experiment placeholder. Should be set by the latest call of Experiment.init()
 experiment = None
 
-DEFAULT_UPLOAD_SIZE_LIMIT = 200 * 1024 * 1024  # 100 MebiBytes
+DEFAULT_UPLOAD_SIZE_LIMIT = 200 * 1024 * 1024  # 200 MebiBytes
 
-DEFAULT_ASSET_UPLOAD_SIZE_LIMIT = 50000 * 1024 * 1024  # 2.5GB
+DEFAULT_ASSET_UPLOAD_SIZE_LIMIT = 100000 * 1024 * 1024  # 100GB
 
 DEFAULT_STREAMER_MSG_TIMEOUT = 60 * 60  # 1 Hour
 
@@ -260,7 +268,7 @@ class Config(object):
         return value
 
     def get_int(self, user_value, config_name, default=None, not_set_value=None):
-        # type: (Any, str, Optional[int], int) -> bool
+        # type: (Any, str, Optional[int], int) -> int
         """
         Returns the correct config value based on the following priority list:
         * User_value if set and not None
@@ -497,7 +505,84 @@ class Config(object):
                 print("%s = %s" % (setting, value))
         print("=" * 65)
 
-    def save(self, directory="./", save_all=False, **kwargs):
+    def get_setting_key(self, setting):
+        # Given a setting short-name, return proper ".comet.config" name
+        # eg, given "api_key" return "comet.api_key"
+        # eg, given "logging_console" return "comet.logging.console"
+        subsections = self.get_subsections()
+        key = None
+        for prefix in subsections:
+            if setting.startswith(prefix + "_"):
+                key = ("comet.%s." % prefix) + setting[len(prefix) + 1 :]
+                break
+        if key is None:
+            key = "comet." + setting
+        return key
+
+    def get_setting_name(self, setting):
+        # Given a setting short-name, return proper env NAME
+        # eg, given "api_key" return "COMET_API_KEY"
+        # eg, given "logging_console" return "COMET_LOGGING_CONSOLE"
+        subsections = self.get_subsections()
+        name = None
+        for prefix in subsections:
+            if setting.startswith(prefix + "_"):
+                name = ("COMET_%s_" % prefix.upper()) + (
+                    setting[len(prefix) + 1 :].upper()
+                )
+                break
+        if name is None:
+            name = "COMET_" + setting.upper()
+        return name
+
+    def validate_value(self, key, value):
+        # type: (str, Any) -> Tuple[bool, str]
+        """
+        Validates and converts value to proper type, or
+        fails.
+
+        Returns a tuple (valid, reason_if_failed)
+        """
+        if key in CONFIG_MAP:
+            if value in [None, ""]:
+                return (False, "invalid value")
+
+            stype = CONFIG_MAP[key]["type"]
+            if stype == "int_list":
+                if not isinstance(value, list) or not all(
+                    [isinstance(v, int) for v in value]
+                ):
+                    return (False, "not all values in list are integers")
+
+            elif not isinstance(value, stype):  # specific type, like bool, int, str
+                return (
+                    False,
+                    "value is wrong type for setting; type `%s` given but type `%s` expected"
+                    % (type(value).__name__, stype.__name__),
+                )
+
+            return (True, "valid")
+
+        else:
+            return (False, "invalid setting")
+
+    def _set_settings(self, settings, environ=False):
+        for setting in settings:
+            key = self.get_setting_key(setting)
+            value = settings[setting]
+            valid, reason = self.validate_value(key, value)
+            if valid:
+                if environ:
+                    name = self.get_setting_name(setting)
+                    os.environ[name] = str(value)
+                else:
+                    self[key] = value
+            else:
+                LOGGER.warning(
+                    "config setting %r failed with value %r: %s", setting, value, reason
+                )
+
+    def save(self, directory="./", save_all=False, force=False, **kwargs):
         """
         Save the settings to .comet.config (default) or
         other path/filename. Defaults are commented out.
@@ -505,10 +590,25 @@ class Config(object):
         Args:
             directory: the path to save the .comet.config config settings.
             save_all: save unset variables with defaults too
+            force: force the file to save if it exists; else don't overwrite
             kwargs: key=value pairs to save
         """
         directory = os.path.expanduser(directory)
         filename = os.path.abspath(os.path.join(directory, ".comet.config"))
+        if os.path.isfile(filename):
+            if not force:
+                LOGGER.error(
+                    "'%s' exists and force is not True; refusing to overwrite", filename
+                )
+                return
+            else:  # try to make a backup copy
+                try:
+                    shutil.copyfile(filename, filename + ".bak")
+                except Exception:
+                    LOGGER.warning(
+                        "Unable to make a backup of config file", exc_info=True
+                    )
+
         print('Saving config to "%s"...' % filename, end="")
         with io.open(filename, "w", encoding="utf-8") as ini_file:
             ini_file.write(six.u("# Config file for Comet.ml\n"))
@@ -641,6 +741,9 @@ CONFIG_MAP = {
     "comet.override_feature.sdk_log_env_variables": {
         "type": bool
     },  # Leave feature toggle default to None
+    "comet.override_feature.artifacts": {
+        "type": bool
+    },  # Leave feature toggle default to None
     # Experiment log controls:
     "comet.auto_log.cli_arguments": {"type": bool},
     "comet.auto_log.code": {"type": bool},
@@ -675,6 +778,7 @@ CONFIG_MAP = {
     "comet.auto_log.tfma": {"type": bool, "default": False},
     # Internals:
     "comet.internal.reporting": {"type": bool, "default": True},
+    "comet.internal.file_upload_worker_ratio": {"type": int, "default": 4},
     # Deprecated:
     "comet.display_summary": {"type": bool, "default": None},
     "comet.auto_log.weights": {"type": bool, "default": None},
@@ -682,7 +786,7 @@ CONFIG_MAP = {
 
 
 def get_config(setting=None):
-    # type: (Any) -> Config
+    # type: (Any) -> Union[Config, Any]
     """
     Get a config or setting from the current config
     (os.environment or .env file).
@@ -742,7 +846,7 @@ def get_previous_experiment(previous_experiment, config):
         return previous_experiment
 
 
-def save(directory="~/", save_all=False, **settings):
+def save(directory="~/", save_all=False, force=False, **settings):
     """
     An easy way to create a config file.
 
@@ -753,6 +857,8 @@ def save(directory="~/", save_all=False, **settings):
         save_all: bool (optional). If True, will create
             entries for all items that are configurable
             with their defaults. Default is False
+        force: bool (optional). If True, overwrite pre-existing
+            .comet.config file. If False, ask.
         settings: any valid setting and value
 
     Valid settings include:
@@ -772,20 +878,144 @@ def save(directory="~/", save_all=False, **settings):
 
     Examples:
 
+    ```python
     >>> import comet_ml
     >>> comet_ml.config.save(api_key="...")
     >>> comet_ml.config.save(api_key="...", directory="./")
+    ```
     """
     cfg = get_config()
-    subsections = cfg.get_subsections()
-    for setting in settings:
-        # Correct the keys:
-        key = None
-        for prefix in subsections:
-            if setting.startswith(prefix + "_"):
-                key = ("comet.%s." % prefix) + setting[len(prefix) + 1 :]
-                break
-        if key is None:
-            key = "comet." + setting
-        cfg[key] = settings[setting]
-    cfg.save(directory, save_all=save_all)
+    cfg._set_settings(settings)
+    cfg.save(directory, save_all=save_all, force=force)
+
+
+def _api_key_save(directory, api_key):
+    """
+    Low-level function to only change the api_key
+    of a .comet.config file in the home directory.
+    """
+    from configobj import ConfigObj
+
+    config_path = os.path.expanduser(os.path.join(directory, ".comet.config"))
+
+    if os.path.exists(config_path):
+        config = ConfigObj(config_path)
+        try:
+            shutil.copyfile(config_path, config_path + ".bak")
+        except Exception:
+            LOGGER.warning("Unable to make a backup of config file", exc_info=True)
+    else:
+        config = ConfigObj()
+        config.filename = config_path
+
+    if "comet" not in config:
+        config["comet"] = {}
+
+    config["comet"]["api_key"] = api_key
+    config.write()
+    LOGGER.info("Comet API key saved in %s", config_path)
+
+
+def init(directory="~/", **settings):
+    # type: (Optional[str], **Any) -> None
+    """
+    An easy, safe, interactive way to set and save your settings.
+
+    Will ask for your api_key if not already set. Your
+    api_key will not be shown.
+
+    Args:
+        directory: str (optional), location to save the
+            .comet.config file. Typical values are "~/" (home)
+            and "./" (current directory). Default is "./"
+        settings: any valid setting and value
+
+    Valid settings include:
+
+    * api_key
+    * disable_auto_logging
+    * experiment_key
+    * offline_directory
+    * workspace
+    * project_name
+    * logging_console
+    * logging_file
+    * logging_file_level
+    * logging_file_overwrite
+    * timeout_cleaning
+    * timeout_upload
+
+    Examples:
+
+    For use in a notebook, or other interactive code:
+
+    ```python
+    >>> import comet_ml
+    >>> comet_ml.init()
+    ```
+
+    You can also provide values for other settings:
+
+    ```python
+    >>> import comet_ml
+    >>> comet_ml.init(project_name='colab')
+    ```
+    """
+    _init(directory=directory, **settings)
+
+
+def _init(directory="~/", __should_prompt_function=None, **settings):
+    # type: (Optional[str], Any, **Any) -> None
+    # We only save the api_key if given by parameter, or
+    # by getpass. We don't save it if in the environment
+    from .api import API
+
+    if __should_prompt_function is None:
+        __should_prompt_function = is_interactive
+
+    config = get_config()
+    need_to_save = False
+    if "api_key" in settings:
+        api_key = settings.pop("api_key", None)
+        if api_key:
+            need_to_save = True
+    else:
+        api_key = config.get_string(None, "comet.api_key")
+
+    if api_key is None:
+        if __should_prompt_function():
+            api_key = get_api_key_from_user()
+            if api_key:
+                need_to_save = True
+
+    failed = False
+    if api_key is None:
+        LOGGER.info(API_KEY_IS_NOT_SET)
+    else:
+        try:
+            API(api_key=api_key).get_workspaces()
+        except InvalidRestAPIKey:
+            failed = True
+            LOGGER.error(
+                API_KEY_IS_INVALID,
+                config.get_string(None, "comet.url_override"),
+                exc_info=True,
+            )
+        except Exception:
+            failed = True
+            LOGGER.error(API_KEY_CHECK_FAILED, exc_info=True)
+        else:
+            LOGGER.info(API_KEY_IS_VALID)
+
+            if need_to_save:
+                try:
+                    _api_key_save(directory, api_key)
+                except Exception:
+                    failed = True
+                    LOGGER.warning(
+                        "Unable to save Comet API key to disk", exc_info=True
+                    )
+
+    if settings and not failed:
+        # Set in environment to save:
+        config._set_settings(settings, environ=True)
