@@ -18,7 +18,12 @@ import random
 from collections import defaultdict
 
 from ._typing import Any, Dict, Iterable, List, Optional, Set, Tuple, Union
-from .convert_utils import convert_tensor_to_numpy, convert_to_list, convert_to_scalar
+from .convert_utils import (
+    convert_tensor_to_numpy,
+    convert_to_bytes,
+    convert_to_list,
+    convert_to_scalar,
+)
 from .logging_messages import (
     CONFUSION_MATRIX_ERROR_RESULTING_LENGTH,
     CONFUSION_MATRIX_ERROR_WRONG_LENGTH,
@@ -91,6 +96,8 @@ class ConfusionMatrix(object):
         index_to_example_function=None,
         cache=True,
         selected=None,
+        images=None,
+        experiment=None,
         **kwargs  # keyword args for index_to_example_function
     ):
         """
@@ -103,6 +110,8 @@ class ConfusionMatrix(object):
             y_predicted: (optional) list of vectors representing predicted
                 values, or a list of integers representing the output. If
                 not provided, then matrix may be provided.
+            images: (optional) a list of data that can be passed to
+                Experiment.log_image()
             labels: (optional) a list of strings that name of the
                 columns and rows, in order. By default, it will be
                 "0" through the number of categories (e.g., rows/columns).
@@ -136,6 +145,8 @@ class ConfusionMatrix(object):
                 greater than max_categories, and selected is not provided,
                 then selected will be computed automatically by selecting
                 the most confused categories.
+            experiment: (optional) passed automatically when using
+                `Experiment.create_confusion_matrix()`.
             kwargs: (optional) any extra keywords and their values will
                 be passed onto the index_to_example_function.
 
@@ -195,6 +206,12 @@ class ConfusionMatrix(object):
 
         https://www.comet.ml/docs/python-sdk/Experiment/#experimentlog_confusion_matrix
         """
+        self.images = images
+        self.experiment = experiment
+        if self.images is not None:
+            if index_to_example_function is None:
+                index_to_example_function = self.image_index_to_example_function
+
         if y_true is not None and y_predicted is not None:
             if matrix is not None:
                 raise ValueError(
@@ -226,11 +243,13 @@ class ConfusionMatrix(object):
         self.use_cache = cache
         self.clear_cache()
         self.clear()
-        self._matrix = None
+        self.images = None
+        self._need_init = True
+        self._example_matrix = {}
         self._dimension = None  # type: Optional[int]
 
         if y_true is not None and y_predicted is not None:
-            self.compute_matrix(y_true, y_predicted, **kwargs)
+            self.compute_matrix(y_true, y_predicted, images=images, **kwargs)
         elif matrix is not None:
             try:
                 self._matrix = convert_to_matrix_dict(matrix)
@@ -241,14 +260,26 @@ class ConfusionMatrix(object):
                     exc_info=True,
                 )
 
+    def need_init(self):
+        """
+        Method to call when you want to reset the confusion matrix.
+        Doesn't reset cache.
+
+        Note: this method is called when the confusion matrix
+            is logged.
+        """
+        self._need_init = True
+
     def clear(self):
         """
         Clear the matrices and type.
         """
+        self._need_init = True
         self.type = None
         self._example_matrix = None
         self._matrix = None
         self._dimension = None  # type: Optional[int]
+        self.images = None
 
     def clear_cache(self):
         """
@@ -263,6 +294,7 @@ class ConfusionMatrix(object):
         """
         Initialize the confusion matrix.
         """
+        self._need_init = False
         self._matrix = {}
         self._example_matrix = {}
 
@@ -333,6 +365,45 @@ class ConfusionMatrix(object):
         """
         return index
 
+    def image_index_to_example_function(self, index, **kwargs):
+        # type: (int, Dict[str, Any]) -> Optional[Dict[str, Any]]
+        """
+        The internal index_to_example function used when
+        passing in images to the compute_matrix() method.
+
+        Args:
+            index: the index of the pattern being tested
+            kwargs: additional keyword arguments for Experiment.log_image()
+
+        Returns:
+            * a dictionary containing keys "sample" and "assetId"
+        """
+        if self.images is None:
+            log_once_at_level(logging.INFO, "images were not set; ignoring examples")
+            return None
+
+        if self.experiment is None:
+            log_once_at_level(
+                logging.INFO,
+                "experiment is not set; use experiment.create_confusion_matrix(); ignoring examples",
+            )
+            return None
+
+        image_array = self.images[index]
+        image_name = "confusion-matrix-%05d.png" % index
+        result = self.experiment.log_image(image_array, name=image_name, **kwargs)
+        if result is None:
+            log_once_at_level(
+                logging.INFO, "unable to generate image from images; ignoring example"
+            )
+            return None
+
+        return {
+            "index": index,
+            "sample": image_name,
+            "assetId": result["imageId"],
+        }
+
     def _set_type_from_example(self, example):
         """
         Take the cached example and set the global
@@ -428,7 +499,8 @@ class ConfusionMatrix(object):
         try:
             example = self.index_to_example_function(index, **kwargs)
         except Exception:
-            LOGGER.error(
+            log_once_at_level(
+                logging.ERROR,
                 CONFUSION_MATRIX_INDEX_TO_EXAMPLE_ERROR,
                 self.index_to_example_function,
                 index,
@@ -440,6 +512,25 @@ class ConfusionMatrix(object):
 
         return example
 
+    def _get_cache_key(self, index):
+        """
+        Given an index, return a globally-unique
+        key for the cache.
+        """
+        if self.images is not None:
+            try:
+                image = self.images[index]
+                array = convert_tensor_to_numpy(image)
+                array_bytes = convert_to_bytes(array)
+                return hash(array_bytes)
+            except Exception:
+                log_once_at_level(
+                    logging.INFO, "unable to generate hash from image", exc_info=True
+                )
+                return None
+        else:
+            return index
+
     def _get_example_from_cache(self, index):
         """
         Get a example from the example cache.
@@ -447,7 +538,7 @@ class ConfusionMatrix(object):
         Args:
             index: the index of example
         """
-        key = index
+        key = self._get_cache_key(index)
         return self._cache_example[key]
 
     def _example_in_cache(self, index):
@@ -457,7 +548,7 @@ class ConfusionMatrix(object):
         Args:
             index: the index of example
         """
-        key = index
+        key = self._get_cache_key(index)
         return key in self._cache_example
 
     def _put_example_in_cache(self, index, example):
@@ -468,7 +559,7 @@ class ConfusionMatrix(object):
             index: the index of example
             example: the processed example
         """
-        key = index
+        key = self._get_cache_key(index)
         self._cache_example[key] = example
 
     def _example_from_list(self, indices, x, y, **kwargs):
@@ -483,31 +574,43 @@ class ConfusionMatrix(object):
             y: the row of example cell
             kwargs: keyword args to pass to user function
         """
-        if len(indices) <= self.max_examples_per_cell:
-            retval = set(indices)
+        key_map = {self._get_cache_key(index): index for index in indices}
+        keys = list(key_map.keys())
+
+        if len(keys) <= self.max_examples_per_cell:
+            retval = keys
         else:
-            indices = list(indices)
-            retval = list(self._cache.intersection(indices))
+            retval = list(self._cache.intersection(keys))
             # If you need more:
             retval += [
-                indices.pop(random.randint(0, len(indices) - 1))
+                keys.pop(random.randint(0, len(keys) - 1))
                 for i in range(self.max_examples_per_cell - len(retval))
             ]
-            # Return minimum needed:
-            retval = set(retval[: self.max_examples_per_cell])
+
+        # Return minimum needed:
+        previous = len(self._example_matrix.get((x, y), []))
+        retval = set(retval[: self.max_examples_per_cell - previous])
 
         new_ones = retval - self._cache
         if self.index_to_example_function is not None:
             examples = []
-            for index in retval:
+            for key in retval:
+                index = key_map[key]
                 example = self._index_to_example(index, **kwargs)
                 if example is not None:
                     examples.append(example)
+
             if len(examples) > 0:
-                self._example_matrix[(x, y)] = examples
+                # Allow to accumulate
+                if (x, y) in self._example_matrix:
+                    self._example_matrix[(x, y)].extend(examples)
+                else:
+                    self._example_matrix[(x, y)] = examples
+
         # Update the ones sent:
         if self.use_cache:
             self._cache.update(new_ones)
+
         return retval
 
     def _get_labels(self):
@@ -563,7 +666,7 @@ class ConfusionMatrix(object):
         return matrix
 
     def compute_matrix(
-        self, y_true, y_predicted, index_to_example_function=None, **kwargs
+        self, y_true, y_predicted, index_to_example_function=None, images=None, **kwargs
     ):
         """
         Compute the confusion matrix.
@@ -573,6 +676,8 @@ class ConfusionMatrix(object):
                 of integers representing the correct label
             y_predicted: list of vectors representing predicted
                 values, or a list of integers representing the output
+            images: (optional) a list of data that can be passed to
+                Experiment.log_image()
             index_to_example_function: (optional) a function
                 that takes an index and returns either
                 a number, a string, a URL, or a {"sample": str,
@@ -584,6 +689,11 @@ class ConfusionMatrix(object):
             Uses winner_function to compute winning categories for
             y_true and y_predicted, if they are vectors.
         """
+        self.images = images
+        if self.images is not None:
+            if index_to_example_function is None:
+                index_to_example_function = self.image_index_to_example_function
+
         if len(y_true) != len(y_predicted):
             raise ValueError(
                 CONFUSION_MATRIX_ERROR_WRONG_LENGTH % (len(y_true), len(y_predicted))
@@ -615,7 +725,8 @@ class ConfusionMatrix(object):
             self.index_to_example_function = index_to_example_function
 
         # Create initial confusion matrix
-        self.initialize()
+        if self._need_init:
+            self.initialize()
 
         examples = defaultdict(set)  # type: Dict[Tuple[int, int], Set[int]]
         for (i, (raw_x, raw_y)) in enumerate(zip(xs, ys)):
@@ -647,7 +758,13 @@ class ConfusionMatrix(object):
         Return the associated confusion matrix as the JSON to
         upload.
         """
-        if (self._matrix is not None) and (self._dimension > self.max_categories):
+        if (
+            (self._matrix is not None)
+            and (self._dimension > self.max_categories)
+            and (self.selected is None)
+        ):
+            # If there is a matrix, and it is bigger than max, and selected is None
+            # then we will automatically select those with most confusion:
             correct_counts = [
                 (i, self._matrix.get((i, i), 0)) for i in range(self._dimension)
             ]
