@@ -25,9 +25,30 @@ from .logging_messages import (
 )
 from .schemas import get_3d_boxes_validator
 
+try:
+    import pathlib
+except ImportError:
+    try:
+        import pathlib2 as pathlib
+    except ImportError:
+        pathlib = None
+
 LOGGER = logging.getLogger(__name__)
 
 
+def convert_to_bytes(item):
+    """
+    Convert the item into bytes. The conversion may not
+    be reversable.
+    """
+    if hasattr(item, "tobytes"):
+        # Numpy arrays, etc.
+        return item.tobytes()
+    else:
+        str_item = str(item)
+        return str_item.encode(encoding="utf-8", errors="xmlcharrefreplace")
+
+
 def convert_tensor_to_numpy(tensor):
     """
     Convert from various forms of pytorch tensors
@@ -228,3 +249,11 @@ def validate_and_convert_3d_boxes(boxes):
         )
 
     return final_boxes
+
+
+def convert_pathlib_path(user_input):
+    # type: (Any) -> Any
+    if pathlib is not None and isinstance(user_input, pathlib.Path):
+        return str(user_input)
+
+    return user_input
