@@ -27,7 +27,6 @@ import platform
 import socket
 import sys
 
-import netifaces
 import requests
 import six
 from six import StringIO
@@ -65,19 +64,6 @@ def get_python_version():
     return platform.python_version()
 
 
-def get_network_interfaces_ips():
-    try:
-        ips = []
-        for interface in netifaces.interfaces():
-            for link in netifaces.ifaddresses(interface).get(netifaces.AF_INET, []):
-                ips.append(link["addr"])
-        return ips
-
-    except Exception:
-        LOGGER.warning("Failed to log all interfaces ips", exc_info=True)
-        return None
-
-
 def get_ip():
     try:
         s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
@@ -129,7 +115,6 @@ def get_env_details():
         "python_version_verbose": get_python_version_verbose(),
         "python_version": get_python_version(),
         "user": get_user(),
-        "network_interfaces_ips": get_network_interfaces_ips(),
         "ip": get_ip(),
         "command": get_command(),
         "python_exe": sys.executable,
@@ -151,7 +136,6 @@ def get_env_details_message(env_blacklist, include_env=False):
         hostname=get_hostname(),
         ip=get_ip(),
         machine=machine,
-        network_interfaces_ips=get_network_interfaces_ips(),
         os_release=os_release,
         os_type=os_type,
         os=get_os(),
