@@ -168,6 +168,55 @@ class InvalidOptimizerParameters(CometException):
     pass
 
 
+class ExperimentDisabledException(CometException):
+    pass
+
+
+class ExperimentNotAlive(CometException):
+    pass
+
+
+class LogArtifactException(CometException):
+    def __init__(self, backend_err_msg=None, sdk_error_code=None):
+        # type: (Optional[str], Optional[int]) -> None
+        self.backend_err_msg = backend_err_msg
+        self.sdk_error_code = sdk_error_code
+
+    def __repr__(self):
+        if self.sdk_error_code is not None and self.backend_err_msg is not None:
+            return "Log artifact failed with an error, %r [%s]" % (
+                self.backend_err_msg,
+                self.sdk_error_code,
+            )
+        elif self.backend_err_msg is not None:
+            return "Log artifact failed with an error, %r" % (self.backend_err_msg)
+        else:
+            return "Log artifact failed with an error, check logs for details"
+
+    def __str__(self):
+        return self.__repr__()
+
+
+class GetArtifactException(CometException):
+    pass
+
+
+class ArtifactNotFound(CometException):
+    pass
+
+
+class ArtifactDownloadException(CometException):
+    pass
+
+
+class ArtifactNotFinalException(CometException):
+    pass
+
+
+class ExperimentCleaningException(CometException):
+    pass
+
+
 class CometRestApiException(CometException):
     def __init__(self, method, response):
         super(CometRestApiException, self).__init__()
@@ -177,21 +226,20 @@ class CometRestApiException(CometException):
             self.safe_json_response = (
                 self.response.json()
             )  # type: Optional[Dict[Any, Any]]
-        except ValueError:
+            self.safe_msg = self.safe_json_response.get("msg", None)  # type: ignore[union-attr]
+            self.sdk_error_code = self.safe_json_response.get("sdk_error_code", None)  # type: ignore[union-attr]
+        except (ValueError, TypeError):
             self.safe_json_response = None
+            self.safe_msg = None
+            self.sdk_error_code = None
 
     def __str__(self):
-        try:
-            msg = self.response.json().get("msg", None)
-        except ValueError:
-            msg = None
-
-        if msg:
+        if self.safe_msg:
             return "%s %s failed with status code %s: %s" % (
                 self.method,
                 self.response.url,
                 self.response.status_code,
-                msg,
+                self.safe_msg,
             )
         else:
             return "%s %s failed with status code %s" % (
