@@ -21,17 +21,13 @@ import logging
 import os
 import shutil
 import tempfile
-import threading
-import zipfile
 
 import six
-from requests import Response
-from requests_toolbelt import MultipartEncoder
 
-from ._reporting import FILE_UPLOADED_FAILED
 from ._typing import (
     IO,
     Any,
+    Callable,
     Dict,
     Optional,
     TemporaryFilePath,
@@ -39,8 +35,7 @@ from ._typing import (
     UserText,
     ValidFilePath,
 )
-from .config import get_config
-from .connection import Reporting, get_http_session
+from .convert_utils import convert_pathlib_path
 from .exceptions import FileIsTooBig
 from .logging_messages import (
     LOG_AUDIO_TOO_BIG,
@@ -52,12 +47,12 @@ from .logging_messages import (
 from .messages import RemoteAssetMessage, UploadFileMessage, UploadInMemoryMessage
 from .utils import (
     data_to_fp,
-    encode_metadata,
     get_file_extension,
     image_data_to_file_like_object,
     write_file_like_to_tmp_file,
     write_numpy_array_as_wav,
 )
+from .validation_utils import validate_metadata
 
 LOGGER = logging.getLogger(__name__)
 
@@ -67,302 +62,10 @@ except ImportError:
     LOGGER.warning("numpy not installed; some functionality will be unavailable")
     pass
 
-
-def compress_git_patch(git_patch):
-    # Create a zip
-    zip_dir = tempfile.mkdtemp()
-
-    zip_path = os.path.join(zip_dir, "patch.zip")
-    archive = zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED)
-    archive.writestr("git_diff.patch", git_patch)
-    archive.close()
-
-    return archive, zip_path
-
-
-def _get_clientlib_params(experiment_id, project_id, api_key):
-    # type: (str, str, str) -> Dict[str, str]
-    return {"experimentId": experiment_id, "projectId": project_id, "apiKey": api_key}
-
-
-def _get_clientlib_headers(experiment_id):
-    # type: (str) -> Dict[str, str]
-    return {"X-COMET-DEBUG-EXPERIMENT-KEY": experiment_id}
-
-
-def _send_file(url, data, params, headers, timeout):
-    # type: (str, Any, Dict[str, str], Dict[str, Any], int) -> Response
-    LOGGER.debug("Uploading files %r to %s with params %s", data, url, params)
-
-    with get_http_session() as session:
-        r = session.post(
-            url, params=params, data=data, timeout=timeout, headers=headers,
-        )  # type: Response
-
-    LOGGER.debug("Uploading file to %s done", url)
-
-    if r.status_code != 200:
-        raise ValueError(
-            "POSTing file failed (%s) on url %r: %r" % (r.status_code, url, r.content)
-        )
-
-    return r
-
-
-def send_file(
-    post_endpoint,
-    api_key,
-    experiment_id,
-    project_id,
-    file_path,
-    params,
-    headers,
-    timeout,
-    metadata=None,
-):
-    # type: (str, str, str, str, str, Dict[str, str], Dict[str, str], int, Optional[Dict[Any, Any]]) -> Response
-    with open(file_path, "rb") as _file:
-        fields = {"file": ("file", _file)}  # type: Dict[str, Any]
-
-        if metadata is not None:
-            encoded_metadata = encode_metadata(metadata)
-            if encoded_metadata:
-                fields["metadata"] = encoded_metadata
-
-        data = MultipartEncoder(fields=fields)
-
-        headers.update({"Content-Type": data.content_type})
-
-        return _send_file(
-            post_endpoint, params=params, data=data, timeout=timeout, headers=headers,
-        )
-
-
-def send_file_like(
-    post_endpoint,
-    api_key,
-    experiment_id,
-    project_id,
-    file_like,
-    params,
-    headers,
-    timeout,
-    metadata=None,
-):
-    # type: (str, str, str, str, Any, Dict[str, str], Dict[str, str], int, Optional[Dict[Any, Any]]) -> Response
-    fields = {"file": ("file", file_like)}  # type: Dict[str, Any]
-
-    if metadata is not None:
-        encoded_metadata = encode_metadata(metadata)
-        if encoded_metadata:
-            fields["metadata"] = encoded_metadata
-
-    data = MultipartEncoder(fields=fields)
-
-    headers.update({"Content-Type": data.content_type})
-
-    return _send_file(
-        post_endpoint, params=params, data=data, timeout=timeout, headers=headers,
-    )
-
-
-def send_remote_asset(
-    post_endpoint,
-    api_key,
-    experiment_id,
-    project_id,
-    remote_uri,
-    params,
-    headers,
-    timeout,
-    metadata=None,
-):
-    # type: (str, str, str, str, str, Dict[str, str], Dict[str, str], int, Optional[Dict[Any, Any]]) -> Response
-
-    fields = {"link": ("link", remote_uri)}  # type: Dict[str, Any]
-
-    if metadata is not None:
-        encoded_metadata = encode_metadata(metadata)
-        if encoded_metadata:
-            fields["metadata"] = encoded_metadata
-
-    data = MultipartEncoder(fields=fields)
-
-    headers.update({"Content-Type": data.content_type})
-
-    return _send_file(
-        post_endpoint, params=params, data=data, timeout=timeout, headers=headers,
-    )
-
-
-def upload_file(
-    project_id,
-    experiment_id,
-    file_path,
-    upload_endpoint,
-    api_key,
-    timeout,
-    additional_params=None,
-    metadata=None,
-    clean=True,
-):
-    params = _get_clientlib_params(experiment_id, project_id, api_key)
-
-    if additional_params is not None:
-        params.update(additional_params)
-
-    headers = _get_clientlib_headers(experiment_id)
-
-    try:
-        response = send_file(
-            upload_endpoint,
-            api_key,
-            experiment_id,
-            project_id,
-            file_path,
-            params=params,
-            headers=headers,
-            metadata=metadata,
-            timeout=timeout,
-        )
-
-        if clean is True:
-            # Cleanup file
-            try:
-                os.remove(file_path)
-            except OSError:
-                pass
-
-        LOGGER.debug(
-            "File successfully uploaded to (%s): %s",
-            response.status_code,
-            upload_endpoint,
-        )
-    except Exception as e:
-        LOGGER.error("File could not be uploaded", exc_info=True)
-        Reporting.report(
-            event_name=FILE_UPLOADED_FAILED,
-            experiment_key=experiment_id,
-            project_id=project_id,
-            api_key=api_key,
-            err_msg=str(e),
-            config=get_config(),
-        )
-
-
-def upload_file_like(
-    project_id,
-    experiment_id,
-    file_like,
-    upload_endpoint,
-    api_key,
-    timeout,
-    additional_params=None,
-    metadata=None,
-):
-    params = _get_clientlib_params(experiment_id, project_id, api_key)
-
-    if additional_params is not None:
-        params.update(additional_params)
-
-    headers = _get_clientlib_headers(experiment_id)
-
-    try:
-        response = send_file_like(
-            upload_endpoint,
-            api_key,
-            experiment_id,
-            project_id,
-            file_like,
-            params=params,
-            headers=headers,
-            metadata=metadata,
-            timeout=timeout,
-        )
-
-        LOGGER.debug(
-            "File-like successfully uploaded to (%s): %s",
-            response.status_code,
-            upload_endpoint,
-        )
-    except Exception as e:
-        LOGGER.error("File-like could not be uploaded", exc_info=True)
-        Reporting.report(
-            event_name=FILE_UPLOADED_FAILED,
-            experiment_key=experiment_id,
-            project_id=project_id,
-            api_key=api_key,
-            err_msg=str(e),
-            config=get_config(),
-        )
-
-
-def upload_remote_asset(
-    project_id,
-    experiment_id,
-    remote_uri,
-    upload_endpoint,
-    api_key,
-    timeout,
-    additional_params=None,
-    metadata=None,
-):
-    params = _get_clientlib_params(experiment_id, project_id, api_key)
-
-    if additional_params is not None:
-        params.update(additional_params)
-
-    headers = _get_clientlib_headers(experiment_id)
-
-    try:
-        response = send_remote_asset(
-            upload_endpoint,
-            api_key,
-            experiment_id,
-            project_id,
-            remote_uri,
-            params=params,
-            headers=headers,
-            metadata=metadata,
-            timeout=timeout,
-        )
-
-        LOGGER.debug(
-            "Remote Asset successfully uploaded to (%s): %s",
-            response.status_code,
-            upload_endpoint,
-        )
-    except Exception as e:
-        LOGGER.error("Remote Asset could not be uploaded", exc_info=True)
-        Reporting.report(
-            event_name=FILE_UPLOADED_FAILED,
-            experiment_key=experiment_id,
-            project_id=project_id,
-            api_key=api_key,
-            err_msg=str(e),
-            config=get_config(),
-        )
-
-
-def upload_file_thread(*args, **kwargs):
-    p = threading.Thread(target=upload_file, args=args, kwargs=kwargs)
-    p.daemon = True
-    p.start()
-    return p
-
-
-def upload_file_like_thread(*args, **kwargs):
-    p = threading.Thread(target=upload_file_like, args=args, kwargs=kwargs)
-    p.daemon = True
-    p.start()
-    return p
-
-
-def upload_remote_asset_thread(*args, **kwargs):
-    p = threading.Thread(target=upload_remote_asset, args=args, kwargs=kwargs)
-    p.daemon = True
-    p.start()
-    return p
+try:
+    from plotly.graph_objects import Figure as PlotlyFigure
+except ImportError:
+    PlotlyFigure = None
 
 
 def is_valid_file_path(file_path):
@@ -390,7 +93,7 @@ def is_user_text(input):
 
 
 def check_max_file_size(file_path, max_upload_size, too_big_msg):
-    # type: (str, int, str) -> None
+    # type: (str, int, str) -> int
     """Check if a file identified by its file path is bigger than the maximum
     allowed upload size. Raises FileIsTooBig if the file is greater than the
     upload limit.
@@ -402,6 +105,8 @@ def check_max_file_size(file_path, max_upload_size, too_big_msg):
         if file_size > max_upload_size:
             raise FileIsTooBig(file_path, file_size, max_upload_size)
 
+        return file_size
+
     except OSError:
         LOGGER.error(too_big_msg, file_path, exc_info=True)
         raise
@@ -441,6 +146,63 @@ def save_matplotlib_figure(figure=None):
         raise TypeError(msg)
 
 
+def save_plotly_figure(figure):
+    # type: (Optional[Any]) -> str
+    """
+    Save the Plotly Figure as an image.
+    """
+    # Save the file to a tempfile but don't delete it, the file uploader
+    # thread will take care of it
+    tmpfile = tempfile.NamedTemporaryFile(suffix=".svg", delete=False)
+    figure.write_image(tmpfile, format="svg")
+    tmpfile.flush()
+    tmpfile.close()
+    return tmpfile.name
+
+
+class AssetUploadUserInput(object):
+    def __init__(self, user_input):
+        self.user_input = user_input
+
+
+class FileUpload(AssetUploadUserInput):
+    def __init__(self, file_path):
+        super(FileUpload, self).__init__(file_path)
+
+        try:
+            self.file_size = os.path.getsize(file_path)
+        except OSError:
+            LOGGER.debug("Error retrieving file size for %r", file_path)
+            self.file_size = 0
+
+
+class MemoryFileUpload(AssetUploadUserInput):
+    pass
+
+
+class UserTextFileUpload(AssetUploadUserInput):
+    pass
+
+
+class ObjectToConvertFileUpload(AssetUploadUserInput):
+    pass
+
+
+def dispatch_user_file_upload(user_input):
+    # type: (Any) -> Union[FileUpload, MemoryFileUpload, UserTextFileUpload, ObjectToConvertFileUpload]
+
+    user_input = convert_pathlib_path(user_input)
+
+    if isinstance(user_input, ValidFilePath) or is_valid_file_path(user_input):
+        return FileUpload(user_input)
+    elif hasattr(user_input, "read"):  # Support Python 2 legacy StringIO
+        return MemoryFileUpload(user_input)
+    elif is_user_text(user_input):
+        return UserTextFileUpload(user_input)
+    else:
+        return ObjectToConvertFileUpload(user_input)
+
+
 class BaseUploadProcessor(object):
 
     TOO_BIG_MSG = ""
@@ -455,14 +217,21 @@ class BaseUploadProcessor(object):
         copy_to_tmp,  # type: bool
         error_message_identifier,  # type: Any
         tmp_dir,  # type: str
+        critical,  # type: bool
+        on_asset_upload=None,
+        on_failed_asset_upload=None,
     ):
         # type: (...) -> None
         self.user_input = user_input
         self.url_params = url_params
-        self.metadata = self._validate_metadata(metadata)
+        self.metadata = validate_metadata(metadata)
         self.upload_limit = upload_limit
         self.error_message_identifier = error_message_identifier
         self.tmp_dir = tmp_dir
+        self.file_size = None  # type: Optional[int]
+        self.critical = critical
+        self.on_asset_upload = on_asset_upload
+        self.on_failed_asset_upload = on_failed_asset_upload
 
         self.copy_to_tmp = copy_to_tmp
 
@@ -470,16 +239,17 @@ class BaseUploadProcessor(object):
 
     def process(self):
         # type: () -> Union[None, UploadInMemoryMessage, UploadFileMessage]
-        if isinstance(self.user_input, ValidFilePath) or is_valid_file_path(
-            self.user_input
-        ):
-            return self.process_upload_by_filepath(self.user_input)
-        elif hasattr(self.user_input, "read"):  # Support Python 2 legacy StringIO
-            return self.process_io_object(self.user_input)
-        elif is_user_text(self.user_input):
-            return self.process_user_text(self.user_input)
+
+        user_input = convert_pathlib_path(self.user_input)
+
+        if isinstance(user_input, ValidFilePath) or is_valid_file_path(user_input):
+            return self.process_upload_by_filepath(user_input)
+        elif hasattr(user_input, "read"):  # Support Python 2 legacy StringIO
+            return self.process_io_object(user_input)
+        elif is_user_text(user_input):
+            return self.process_user_text(user_input)
         else:
-            return self.process_upload_to_be_converted(self.user_input)
+            return self.process_upload_to_be_converted(user_input)
 
     # Dispatched user input method, one method per supported type in general. By
     # default those methods raise an exception, implement them for supported
@@ -506,7 +276,9 @@ class BaseUploadProcessor(object):
     def _process_upload_by_filepath(self, user_filepath):
         # type: (ValidFilePath) -> Optional[UploadFileMessage]
         try:
-            check_max_file_size(user_filepath, self.upload_limit, self.TOO_BIG_MSG)
+            self.file_size = check_max_file_size(
+                user_filepath, self.upload_limit, self.TOO_BIG_MSG
+            )
         except FileIsTooBig as exc:
             if self.error_message_identifier is None:
                 error_message_identifier = exc.file_path
@@ -552,6 +324,9 @@ class BaseUploadProcessor(object):
             self.url_params,
             self.metadata,
             clean=clean,
+            critical=self.critical,
+            on_asset_upload=self.on_asset_upload,
+            on_failed_asset_upload=self.on_failed_asset_upload,
         )
 
         return upload_message
@@ -603,7 +378,13 @@ class BaseUploadProcessor(object):
         )
 
         return UploadInMemoryMessage(
-            io_object, self.UPLOAD_TYPE, self.url_params, self.metadata
+            io_object,
+            self.UPLOAD_TYPE,
+            self.url_params,
+            self.metadata,
+            critical=self.critical,
+            on_asset_upload=self.on_asset_upload,
+            on_failed_asset_upload=self.on_failed_asset_upload,
         )
 
     def _process_upload_text(self, user_text):
@@ -628,24 +409,15 @@ class BaseUploadProcessor(object):
         )
 
         return UploadInMemoryMessage(
-            user_text, self.UPLOAD_TYPE, self.url_params, self.metadata
+            user_text,
+            self.UPLOAD_TYPE,
+            self.url_params,
+            self.metadata,
+            critical=self.critical,
+            on_asset_upload=self.on_asset_upload,
+            on_failed_asset_upload=self.on_failed_asset_upload,
         )
 
-    @staticmethod
-    def _validate_metadata(user_metadata):
-        # type: (Any) -> Dict[Any, Any]
-
-        if user_metadata is None:
-            return {}
-
-        if type(user_metadata) is not dict:
-            LOGGER.warning("Invalid metadata, expecting dict type", exc_info=True)
-            return {}
-
-        result = user_metadata  # type: Dict[Any, Any]
-
-        return result
-
 
 class AssetUploadProcessor(BaseUploadProcessor):
 
@@ -661,15 +433,13 @@ class AssetUploadProcessor(BaseUploadProcessor):
         copy_to_tmp,  # type: bool
         error_message_identifier,  # type: Any
         tmp_dir,  # type: str
+        critical,  # type: bool
+        on_asset_upload=None,
+        on_failed_asset_upload=None,
     ):
         # type: (...) -> None
         self.UPLOAD_TYPE = upload_type
 
-        if metadata is None:
-            self.metadata = {}
-        else:
-            self.metadata = metadata
-
         super(AssetUploadProcessor, self).__init__(
             user_input,
             upload_limit,
@@ -678,6 +448,9 @@ class AssetUploadProcessor(BaseUploadProcessor):
             copy_to_tmp,
             error_message_identifier,
             tmp_dir,
+            critical,
+            on_asset_upload,
+            on_failed_asset_upload,
         )
 
     def process_upload_by_filepath(self, upload_filepath):
@@ -718,8 +491,12 @@ class FigureUploadProcessor(BaseUploadProcessor):
         copy_to_tmp,  # type: bool
         error_message_identifier,  # type: Any
         tmp_dir,  # type: str
+        critical,  # type: bool
         upload_type=None,  # type: Optional[str]
+        on_asset_upload=None,
+        on_failed_asset_upload=None,
     ):
+        # type: (...) -> None
         super(FigureUploadProcessor, self).__init__(
             user_input,
             upload_limit,
@@ -728,18 +505,33 @@ class FigureUploadProcessor(BaseUploadProcessor):
             copy_to_tmp,
             error_message_identifier,
             tmp_dir,
+            critical,
+            on_asset_upload,
+            on_failed_asset_upload,
         )
         if upload_type is not None:
             self.UPLOAD_TYPE = upload_type
 
     def process_upload_to_be_converted(self, user_input):
         # type: (Any) -> Optional[UploadFileMessage]
-        try:
-            filename = save_matplotlib_figure(user_input)
-        except Exception:
-            LOGGER.warning("Failing to save the matplotlib figure", exc_info=True)
-            # An error occurred
-            return None
+        if PlotlyFigure is not None and isinstance(user_input, PlotlyFigure):
+            try:
+                filename = save_plotly_figure(user_input)
+            except Exception:
+                LOGGER.warning(
+                    "Failing to save the plotly figure; requires dependencies: see https://plotly.com/python/static-image-export/",
+                    exc_info=True,
+                )
+                # An error occurred
+                return None
+
+        else:
+            try:
+                filename = save_matplotlib_figure(user_input)
+            except Exception:
+                LOGGER.warning("Failing to save the matplotlib figure", exc_info=True)
+                # An error occurred
+                return None
 
         self.url_params["extension"] = get_file_extension(filename)
 
@@ -768,6 +560,9 @@ class ImageUploadProcessor(BaseUploadProcessor):
         copy_to_tmp,  # type: bool
         error_message_identifier,  # type: Any
         tmp_dir,
+        critical,  # type: bool
+        on_asset_upload=None,  # type: Callable
+        on_failed_asset_upload=None,  # type: Callable
     ):
         # type: (...) -> None
         self.name = name
@@ -785,6 +580,9 @@ class ImageUploadProcessor(BaseUploadProcessor):
             copy_to_tmp,
             error_message_identifier,
             tmp_dir,
+            critical,
+            on_asset_upload,
+            on_failed_asset_upload,
         )
 
     def process_upload_by_filepath(self, upload_filepath):
@@ -854,15 +652,13 @@ class AudioUploadProcessor(BaseUploadProcessor):
         copy_to_tmp,  # type: bool
         error_message_identifier,  # type: Any
         tmp_dir,  # type: str
+        critical,  # type: bool
+        on_asset_upload=None,  # type: Callable
+        on_failed_asset_upload=None,  # type: Callable
     ):
         # type: (...) -> None
         self.sample_rate = sample_rate
 
-        if metadata is None:
-            self.metadata = {}
-        else:
-            self.metadata = metadata
-
         super(AudioUploadProcessor, self).__init__(
             user_input,
             upload_limit,
@@ -871,6 +667,9 @@ class AudioUploadProcessor(BaseUploadProcessor):
             copy_to_tmp,
             error_message_identifier,
             tmp_dir,
+            critical,
+            on_asset_upload,
+            on_failed_asset_upload,
         )
 
     def process_upload_by_filepath(self, upload_filepath):
@@ -948,6 +747,9 @@ class AssetDataUploadProcessor(BaseUploadProcessor):
         copy_to_tmp,  # type: bool
         error_message_identifier,  # type: Any
         tmp_dir,  # type: str
+        critical,  # type: bool
+        on_asset_upload=None,  # type: Callable
+        on_failed_asset_upload=None,  # type: Callable
     ):
         # type: (...) -> None
         self.UPLOAD_TYPE = upload_type
@@ -959,6 +761,9 @@ class AssetDataUploadProcessor(BaseUploadProcessor):
             copy_to_tmp,
             error_message_identifier,
             tmp_dir,
+            critical,
+            on_asset_upload,
+            on_failed_asset_upload,
         )
 
     def process_upload_to_be_converted(self, user_input):
@@ -1002,11 +807,19 @@ class RemoteAssetUploadProcessor(object):
         upload_type,  # type: str
         url_params,  # type: Dict[str, Optional[Any]]
         metadata,  # type: Optional[Dict[str, str]]
+        critical,  # type: bool
+        on_asset_upload=None,
+        on_failed_asset_upload=None,
     ):
+        # type: (...) -> None
         self.remote_uri = remote_uri
         self.upload_type = upload_type
         self.url_params = url_params
         self.metadata = metadata
+        self.critical = critical
+
+        self.on_asset_upload = on_asset_upload
+        self.on_failed_asset_upload = on_failed_asset_upload
 
     def process(self):
         # type: () -> RemoteAssetMessage
@@ -1014,5 +827,11 @@ class RemoteAssetUploadProcessor(object):
         self.url_params.update({"isRemote": True})
 
         return RemoteAssetMessage(
-            self.remote_uri, self.upload_type, self.url_params, self.metadata
+            self.remote_uri,
+            self.upload_type,
+            self.url_params,
+            self.metadata,
+            self.critical,
+            on_asset_upload=self.on_asset_upload,
+            on_failed_asset_upload=self.on_failed_asset_upload,
         )
