@@ -15,9 +15,6 @@ import logging
 import tempfile
 from os.path import basename, join
 
-from comet_ml._logging import check_module
-from comet_ml.offline import OfflineExperiment
-
 from .._typing import Optional
 from ..config import get_config, get_global_experiment, set_global_experiment
 from ..experiment import BaseExperiment
@@ -26,6 +23,8 @@ from ..logging_messages import (
     MLFLOW_OFFLINE_EXPERIMENT_FALLBACK,
     MLFLOW_RESUMED_RUN,
 )
+from ..monkey_patching import check_module
+from ..offline import OfflineExperiment
 
 LOGGER = logging.getLogger(__name__)
 
