@@ -15,8 +15,8 @@
 
 import logging
 
-from .._logging import check_module
 from .._typing import Any, Dict, List, Tuple
+from ..monkey_patching import check_module
 from ..utils import Histogram
 
 LOGGER = logging.getLogger(__name__)
