@@ -16,7 +16,7 @@
 import json
 import logging
 
-from .._logging import check_module
+from ..monkey_patching import check_module
 from ..utils import data_to_fp
 
 LOGGER = logging.getLogger(__name__)
