@@ -241,3 +241,48 @@ LOG_PARAMS_EMPTY_MAPPING = "Empty mapping given to log_params(%r); ignoring"
 LOG_PARAMS_EMPTY_CONVERTED_MAPPING = (
     "%r passed to log_params converted to an empty maping; ignoring"
 )
+
+API_KEY_IS_NOT_SET = "Comet API key is not set. You'll have to provide it explictly to experiments and API"
+
+API_KEY_IS_VALID = "Comet API key is valid"
+
+API_KEY_IS_INVALID = (
+    "Invalid Comet API key for %s\n"
+    + "You will not be able to create online Experiments\n"
+    + "Please see https://www.comet.ml/docs/command-line/#comet-check for more information.\n"
+    + "Use: comet_ml.init() to try again"
+)
+
+API_KEY_CHECK_FAILED = "Unable to verify Comet API key at this time"
+
+JUPYTER_NEEDS_END = "As you are running in a Jupyter environment, you will need to call `experiment.end()` when finished to ensure all metrics and code are logged before exiting."
+
+ARTIFACT_VERSION_CREATED_WITH_PREVIOUS = (
+    "Artifact %r version %s created (previous was: %s)"
+)
+
+ARTIFACT_VERSION_CREATED_WITHOUT_PREVIOUS = "Artifact %r version %s created"
+
+GET_ARTIFACT_WORKSPACE_GIVEN_TWICE = "Workspace was given both explicitly %r and as part of the fully-qualified artifact name %r, using the explicit value"
+
+GET_ARTIFACT_VERSION_OR_ALIAS_GIVEN_TWICE = "Version_or_alias was given both explicitly %r and as part of the fully-qualified artifact name %r, using the explicit value"
+
+ARTIFACT_DOWNLOAD_FILE_OVERWRITTEN = (
+    "File %r has been overwritten by asset %s of artifact version %s"
+)
+
+ARTIFACT_DOWNLOAD_FILE_PRESERVED = 'File %r is different from asset %s of artifact version %s, but was kept because overwrite_strategy="PRESERVE" was set'
+
+ARTIFACT_UPLOAD_STARTED = "Artifact '%s/%s:%s' has started uploading asynchronously"
+
+ARTIFACT_UPLOAD_FINISHED = "Artifact '%s/%s:%s' has been fully uploaded successfully"
+
+ARTIFACT_ASSET_UPLOAD_FAILED = "Asset %r of artifact '%s/%s:%s' upload failed"
+
+ARTIFACT_PREVIEW_FEATURE_FLAG = (
+    "Artifact is currently a preview feature, contact us to get access to it."
+)
+
+LOG_ARTIFACT_IN_PROGESS_MESSAGE = (
+    "Still scheduling the upload of %d assets, remaining size %s"
+)
