@@ -38,7 +38,7 @@ def check_module(module):
     This is an error, unless comet.disable_auto_logging == 1
     """
     if get_config("comet.disable_auto_logging"):
-        LOGGER.debug(COMET_DISABLED_AUTO_LOGGING_MSG % module)
+        LOGGER.debug(COMET_DISABLED_AUTO_LOGGING_MSG, module)
     elif module in sys.modules:
         ALREADY_IMPORTED_MODULES.add(module)
 
