@@ -24,7 +24,6 @@ import logging
 import os.path
 import shutil
 import tempfile
-import threading
 import traceback
 import zipfile
 from os.path import join
@@ -37,11 +36,12 @@ from ._reporting import (
     OFFLINE_INVALID_UPLOAD_MSG,
     OFFLINE_INVALID_WS_MSG,
 )
-from ._typing import List, Optional
+from ._typing import Optional
 from .comet import OfflineStreamer, format_url, generate_guid, is_valid_experiment_key
 from .compat_utils import json_dump
 from .config import get_api_key, get_config, get_previous_experiment
 from .connection import (
+    FileUploadManager,
     RestServerConnection,
     WebSocketConnection,
     format_messages_for_ws,
@@ -56,7 +56,6 @@ from .exceptions import (
 )
 from .experiment import BaseExperiment
 from .feature_toggles import FeatureToggles
-from .file_uploader import upload_file_thread, upload_remote_asset_thread
 from .logging_messages import (
     CLOUD_DETAILS_MSG_SENDING_ERROR,
     MODEL_GRAPH_MSG_SENDING_ERROR,
@@ -511,7 +510,10 @@ class OfflineSender(object):
         self.ws_connection = None
         self.rest_api_client = None
         self.focus_link = None
-        self.upload_threads = []  # type: List[threading.Thread]
+
+        self.file_upload_manager = FileUploadManager(
+            self.config.get_int(None, "comet.internal.file_upload_worker_ratio")
+        )
 
         self.validation_error = validation_error
 
@@ -690,7 +692,7 @@ class OfflineSender(object):
         additional_params = message["additional_params"] or {}
         additional_params["runId"] = self.run_id
 
-        upload_thread = upload_file_thread(
+        self.file_upload_manager.upload_file_thread(
             project_id=self.project_id,
             experiment_id=self.experiment_id,
             file_path=join(self.offline_dir, message["file_path"]),
@@ -700,9 +702,7 @@ class OfflineSender(object):
             clean=True,
             timeout=self.config.get_int(None, "comet.timeout.file_upload"),
         )
-        self.upload_threads.append(upload_thread)
         LOGGER.debug("Processing uploading message done")
-        LOGGER.debug("Upload threads %s", self.upload_threads)
 
     def _process_remote_file_message(self, message):
         message = message["payload"]
@@ -723,7 +723,7 @@ class OfflineSender(object):
         additional_params = message["additional_params"] or {}
         additional_params["runId"] = self.run_id
 
-        upload_thread = upload_remote_asset_thread(
+        self.file_upload_manager.upload_remote_asset_thread(
             project_id=self.project_id,
             experiment_id=self.experiment_id,
             remote_uri=message["remote_uri"],
@@ -733,9 +733,7 @@ class OfflineSender(object):
             metadata=message["metadata"],
             timeout=self.config.get_int(None, "comet.timeout.file_upload"),
         )
-        self.upload_threads.append(upload_thread)
-        LOGGER.debug("Processing uploading message done")
-        LOGGER.debug("Upload threads %s", self.upload_threads)
+        LOGGER.debug("Processing remote uploading message done")
 
     def _process_os_packages_message(self, message):
         message = message["payload"]
@@ -900,9 +898,9 @@ class OfflineSender(object):
         if self.rest_api_client is not None:
             self.rest_api_client.close()
 
-        for thread in self.upload_threads:
-            thread.join()
-        LOGGER.debug("Upload threads %r", self.upload_threads)
+        self.file_upload_manager.close()
+        self.file_upload_manager.join()
+        LOGGER.debug("Upload threads %r", self.file_upload_manager)
 
         LOGGER.log(self.display_level, OFFLINE_SENDER_ENDS, self._get_experiment_url())
 
