@@ -154,8 +154,8 @@ class Optimizer(object):
             self.id = os.environ.get("COMET_OPTIMIZER_ID")
             if verbose > 0:
                 LOGGER.info(
-                    "Using Optimizer id '%s' overridden from environment, ignoring local optimizer config"
-                    % self.id
+                    "Using Optimizer id '%s' overridden from environment, ignoring local optimizer config",
+                    self.id,
                 )
         elif config is None:
             raise OptimizerException(
@@ -183,7 +183,7 @@ class Optimizer(object):
         if "predictor" in status and status["predictor"]:
             self.predictor_spec = self._fill_defaults_predictor(status["predictor"])
         elif verbose > 0:
-            LOGGER.info("Using optimizer config: %s" % status)
+            LOGGER.info("Using optimizer config: %s", status)
 
     def get_experiments(self, **kwargs):
         """
@@ -381,8 +381,8 @@ class Optimizer(object):
         else:
             result = "aborted"
             LOGGER.info(
-                "Optimizer metrics is '%s' but no logged values found. Experiment ignored in sweep."
-                % metric
+                "Optimizer metrics is '%s' but no logged values found. Experiment ignored in sweep.",
+                metric,
             )
         self._api.optimizer_update(self.id, pid, trial, result, score)
         # Log optimizer results with the experiment:
@@ -555,7 +555,7 @@ class Optimizer(object):
                     % (key, status[key], request[key])
                 )
         if verbose > 0:
-            LOGGER.info("Using optimizer config: %s" % status)
+            LOGGER.info("Using optimizer config: %s", status)
 
     def _complete_spec(self, spec, algorithm):
         """
