@@ -56,7 +56,9 @@ def check_ws_connection(websocket_url, debug):
     # type: (str, bool) -> bool
 
     try:
-        http_proxy_host, http_proxy_port, http_proxy_auth, proxy_type = get_http_proxy()
+        http_proxy_host, http_proxy_port, http_proxy_auth, proxy_type = get_http_proxy(
+            websocket_url
+        )
         ws = websocket.create_connection(
             websocket_url,
             http_proxy_host=http_proxy_host,
@@ -99,7 +101,7 @@ def get_default_ws_url(clientlib_address):
 def check_server_connection(server_address):
     url = get_run_id_url(server_address)
     try:
-        with get_http_session(server_address) as session:
+        with get_http_session() as session:
             payload = {
                 "apiKey": "XXX",
                 "local_timestamp": local_timestamp(),
@@ -120,7 +122,7 @@ def check_server_connection(server_address):
 def check_rest_api_connection(rest_api_url):
     url = url_join(rest_api_url, "account-details")
     try:
-        with get_http_session(rest_api_url) as session:
+        with get_http_session() as session:
             session.get(url)
             return True
     except Exception:
@@ -131,7 +133,7 @@ def check_rest_api_connection(rest_api_url):
 def check_optimizer_connection(optimizer_url):
     url = url_join(optimizer_url, "spec")
     try:
-        with get_http_session(optimizer_url) as session:
+        with get_http_session() as session:
             params = {"algorithmName": "bayes"}
             session.get(url, params=params)
             return True
@@ -140,17 +142,6 @@ def check_optimizer_connection(optimizer_url):
         return False
 
 
-def check_predictor_connection(predictor_url):
-    url = url_join(predictor_url, "isAlive/ping")
-    try:
-        with get_http_session(predictor_url) as session:
-            session.get(url)
-            return True
-    except Exception:
-        LOGGER.error("Error checking predictor connectivity", exc_info=True)
-        return False
-
-
 def config_source(env):
     # type: (Any) -> str
     if isinstance(env, ConfigOSEnv):
@@ -277,32 +268,6 @@ def check(args, rest=None):
     else:
         LOGGER.warning("Optimizer connection is not ok")
 
-    # Predictor
-    LOGGER.info("=" * 80)
-    LOGGER.info("Checking connectivity to Predictor Server")
-    LOGGER.info("=" * 80)
-
-    predictor_url = sanitize_url(config["comet.predictor_url"])
-    predictor_url_config_origin = config_source(
-        config.get_config_origin("comet.predictor_url")
-    )
-    LOGGER.info(
-        "Configured Predictor address %r", predictor_url,
-    )
-    if predictor_url_config_origin:
-        LOGGER.info(
-            "Predictor address was configured in %s", predictor_url_config_origin
-        )
-    else:
-        LOGGER.info("Predictor address is the default one")
-    print("")
-    predictor_connected = check_predictor_connection(predictor_url)
-    print("")
-    if predictor_connected:
-        LOGGER.info("Predictor connection is ok")
-    else:
-        LOGGER.warning("Predictor connection is not ok")
-
     print("")
     print("")
 
@@ -312,7 +277,6 @@ def check(args, rest=None):
     LOGGER.info("Rest API connectivity\t\t%r", rest_api_connected)
     LOGGER.info("WS server connectivity\t\t%r", ws_connected)
     LOGGER.info("Optimizer server connectivity\t%r", optimizer_connected)
-    LOGGER.info("Predictor server connectivity\t%r", predictor_connected)
 
 
 def get_parser_arguments(parser):
