@@ -14,8 +14,24 @@
 # *******************************************************
 
 """
-Creates a new project directory with a Python script and dependency file that
-shows how to incorporate Comet with various ML libraries.
+You can use `comet init` to:
+
+1. create a Comet configuration file with your API key; OR
+2. create a new project directory with sample code based on a template
+
+You may wish to do both in this order.
+
+The first is used in this manner in the terminal:
+
+$ comet init --api-key
+
+This will ask you for your Comet API key.
+
+The second is used to create a new project directory with a Python
+script and dependency file that shows how to incorporate Comet with
+various ML libraries. It is called like:
+
+$ comet init
 """
 
 from __future__ import print_function
@@ -25,13 +41,21 @@ import logging
 import os
 import sys
 
-from comet_ml.config import get_config
+from comet_ml.config import _init as init_api_key, get_config
 
 LOGGER = logging.getLogger("comet_ml")
 ADDITIONAL_ARGS = True
 
 
 def get_parser_arguments(parser):
+    parser.add_argument(
+        "-a",
+        "--api-key",
+        action="store_const",
+        const=True,
+        default=False,
+        help="Create a ~/.config.comet file with Comet API key",
+    )
     parser.add_argument(
         "-l",
         "--language",
@@ -75,13 +99,20 @@ def main(args):
 
 
 def init(parsed_args, remaining):
+    if parsed_args.api_key:
+        init_api_key(__should_prompt_function=lambda: sys.stdin.isatty())
+    else:
+        init_cookiecutter(parsed_args)
+
+
+def init_cookiecutter(parsed_args):
     try:
         from cookiecutter.main import cookiecutter
         from cookiecutter.exceptions import OutputDirExistsException
         from click.exceptions import Abort
     except ImportError:
         LOGGER.error(
-            "Please install cookiecutter with `pip install cookiecutter>1.7.0`"
+            'Please install cookiecutter with `pip install "cookiecutter>1.7.0"`'
         )
         sys.exit(1)
 
