@@ -23,7 +23,9 @@ import math
 import numbers
 import os
 import os.path
+import random
 import re
+import sys
 import tempfile
 import time
 from datetime import datetime
@@ -31,6 +33,7 @@ from datetime import datetime
 import six
 from pkg_resources import DistributionNotFound, get_distribution
 from requests.models import PreparedRequest
+from six.moves.urllib.parse import urlparse
 
 from ._typing import (
     IO,
@@ -118,6 +121,7 @@ def log_once_at_level(logging_level, message, *args, **kwargs):
 
 
 def merge_url(url, params):
+    # type: (str, Dict[Any, Any]) -> str
     """
     Given an URL that might have query strings,
     combine with additional query strings.
@@ -334,8 +338,8 @@ def array_to_image(
     Convert a numpy array to an in-memory image.
     """
     try:
-        import PIL.Image
         import numpy
+        import PIL.Image
         from matplotlib import cm
     except ImportError:
         LOGGER.error(
@@ -364,18 +368,28 @@ def array_to_image(
     if image_shape is not None:
         try:
             ## array shape is opposite of image size(width, height)
-            array = array.reshape(image_shape[1], image_shape[0])
+            if len(image_shape) == 2:
+                array = array.reshape(image_shape[1], image_shape[0])
+            elif len(image_shape) == 3:
+                array = array.reshape(image_shape[1], image_shape[0], image_shape[2])
+            else:
+                raise Exception(
+                    "invalid image_shape: %s; should be 2D or 3D" % image_shape
+                )
         except Exception:
             LOGGER.info("WARNING: invalid image_shape; ignored", exc_info=True)
 
+    if image_channels == "first" and len(array.shape) == 3:
+        array = numpy.moveaxis(array, 0, -1)
     ## If 3D, but last array is flat, make it 2D:
-    if len(array.shape) == 3 and array.shape[-1] == 1:
-        array = array.reshape((array.shape[0], array.shape[1]))
+    if len(array.shape) == 3:
+        if array.shape[-1] == 1:
+            array = array.reshape((array.shape[0], array.shape[1]))
+        elif array.shape[0] == 1:
+            array = array.reshape((array.shape[1], array.shape[2]))
     elif len(array.shape) == 1:
         ## if 1D, make it 2D:
         array = numpy.array([array])
-    if image_channels == "first" and len(array.shape) == 3:
-        array = numpy.moveaxis(array, 0, -1)
 
     ### Ok, now let's colorize and scale
     if image_colormap is not None:
@@ -510,8 +524,8 @@ def image_transparent_color(image, color, threshold=1):
         threshold: the max difference in each color component
     """
     try:
-        from PIL import Image
         import numpy
+        from PIL import Image
     except ImportError:
         LOGGER.error(
             "The Python libraries PIL and numpy are required for this operation"
@@ -908,11 +922,11 @@ class Histogram(object):
             for i, match in enumerate(negative_values_filter):
                 if match:
                     self.counts[negative_indices[i]] += counts[i]
-            self.counts[midpoint] += counts[zero_values]
         else:
             numpy.add.at(self.counts, positive_indices, 1)
             numpy.add.at(self.counts, negative_indices, 1)
-            self.counts[midpoint] += len(zero_values)
+
+        self.counts[midpoint] += len(zero_values)
 
     def add(self, values, counts=None, max_skip_count=None):
         """
@@ -1290,7 +1304,7 @@ def get_file_extension(file_path):
 
 
 def encode_metadata(metadata):
-    # type: (Optional[Dict[Any, Any]]) -> Optional[Union[str, bytes]]
+    # type: (Optional[Dict[Any, Any]]) -> Optional[str]
     if metadata is None:
         return None
 
@@ -1303,8 +1317,7 @@ def encode_metadata(metadata):
 
     try:
         json_encoded = json.dumps(metadata, separators=(",", ":"), sort_keys=True)
-        encoded = json_encoded.encode("utf-8")
-        return encoded
+        return json_encoded
     except Exception:
         LOGGER.info("invalid metadata, expecting JSON-encodable object", exc_info=True)
         return None
@@ -1393,49 +1406,48 @@ def valid_ui_tabs(tab=None, preferred=False):
     """
     List of valid UI tabs in browser.
     """
-    preferred_names = [
-        "assets",
-        "audio",
-        "charts",
-        "code",
-        "confusion-matrices",
-        "histograms",
-        "images",
-        "installed-packages",
-        "metrics",
-        "notes",
-        "parameters",
-        "system-metrics",
-        "text",
-    ]
     mappings = {
-        "asset": "assetStorage",
-        "assetStorage": "assetStorage",
-        "assets": "assetStorage",
+        "artifacts": "artifacts",
+        "assets": "assets",
         "audio": "audio",
-        "chart": "chart",
         "charts": "chart",
         "code": "code",
         "confusion-matrices": "confusionMatrix",
-        "confusion-matrix": "confusionMatrix",
-        "confusionMatrix": "confusionMatrix",
         "graphics": "images",
         "histograms": "histograms",
-        "images": "images",
         "installed-packages": "installedPackages",
-        "installedPackages": "installedPackages",
         "metrics": "metrics",
         "notes": "notes",
         "parameters": "params",
-        "params": "params",
         "system-metrics": "systemMetrics",
-        "systemMetrics": "systemMetrics",
         "text": "text",
+        "output": "stdout",
+        "panels": "panels",
+        "graph": "graph",
+        "other": "other",
+        "html": "html",
     }
+    preferred_names = list(mappings.keys())
+    # Additional keys:
+    mappings.update(
+        {
+            "assetStorage": "assets",
+            "chart": "chart",
+            "confusion-matrix": "confusionMatrix",
+            "confusionMatrix": "confusionMatrix",
+            "images": "images",
+            "installedPackages": "installedPackages",
+            "params": "params",
+            "systemMetrics": "systemMetrics",
+            "graph-def": "graph",
+            "graph-definition": "graph",
+            "stdout": "stdout",
+        }
+    )
     if preferred:
         return preferred_names
     elif tab is None:
-        return mappings.keys()
+        return list(mappings.keys())
     elif tab in mappings:
         return mappings[tab]
     else:
@@ -1682,6 +1694,13 @@ def proper_registry_model_name(name):
     return name
 
 
+def is_interactive():
+    """
+    Returns True if in interactive mode
+    """
+    return bool(getattr(sys, "ps1", sys.flags.interactive))
+
+
 def safe_filename(filename):
     """
     Given a value, turn it into a valid filename.
@@ -1820,3 +1839,73 @@ def prepare_dataframe(dataframe, asset_format, **kwargs):
 
     fp.seek(0)
     return fp
+
+
+def get_api_key_from_user():
+    """
+    Get the Comet API key fromthe user.
+    """
+    print("Please enter your Comet API key from https://www.comet.ml/api/my/settings")
+    print("(api key may not show as you type)")
+    api_key = getpass.getpass("Comet API key: ")
+    api_key = "".join([char for char in api_key if char not in ["'", '"', " "]])
+    return api_key
+
+
+def parse_remote_uri(uri):
+    # type: (Optional[str]) -> Optional[str]
+    if not uri:
+        return None
+
+    try:
+        parsed = urlparse(uri)
+        if parsed.path:
+            # Split the path
+            return parsed.path.split("/")[-1]
+        else:
+            return None
+    except Exception:
+        LOGGER.debug("Failure parsing URI %r", uri, exc_info=True)
+        return None
+
+
+def make_template_filename(group=None):
+    if group is None:
+        group = random.randint(0, 10000000)
+
+    return "template_projector_config-%s.json" % group
+
+
+class ImmutableDict(dict):
+    def _immutable(self, *args, **kws):
+        raise TypeError("object is immutable")
+
+    __setitem__ = _immutable
+    __delitem__ = _immutable
+    clear = _immutable
+    update = _immutable
+    setdefault = _immutable
+    pop = _immutable
+    popitem = _immutable
+
+
+class IterationProgressCallback(object):
+    def __init__(self, iterable, callback, frequency):
+        self._iterable = iterable
+        self._callback = callback
+        self._frequency = frequency
+
+    def __iter__(self):
+        last = get_time_monotonic()
+
+        for obj in self._iterable:
+            yield obj
+
+            now = get_time_monotonic()
+            diff = now - last
+            if diff > self._frequency:
+                last = now
+                try:
+                    self._callback()
+                except Exception:
+                    LOGGER.debug("Error calling the progress callback", exc_info=True)
